@@ -14,6 +14,13 @@ import urllib3
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
+    def random_useragent() :
+        ntvers = ["9.0","10.0","8.0"]
+        os = [ "Windows NT "+str(v) for v in ntvers ] + ["Macintosh"] * 3
+        firevers  = [str(i)+".0" for i in range(42,51)]
+        o = random.choice(os)
+        f = random.choice(firevers)
+        return "Mozilla/5.0 ({0}; Win64; x64; rv:{1}) Gecko/20100101 Firefox/{1}".format(o,f)
     ss = requests.Session()
     tgtdir = os.path.expanduser(tgtdir)
     if not os.path.isdir(tgtdir) :
@@ -38,7 +45,11 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
             if downloaded :
                 last = max(downloaded)
     try :
-        rsp = ss.get(page,timeout=20,verify=False)
+        user_agent = {'User-Agent': random_useragent()}
+        if debug :
+            print("# {}".format(page),file=sys.stderr,flush=True)
+            print("# {}".format(user_agent),file=sys.stderr,flush=True)
+        rsp = ss.get(page,headers=user_agent,timeout=20,verify=False)
     except :
         print("# {}".format(traceback.format_exc().splitlines()[-1]),file=sys.stderr,flush=True)
         return
@@ -48,10 +59,20 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         print(rtext)
     chapters = list()
     for m in re.findall(r"(<dd>|<li>)\s*<a\s+href=\"(\S*?\d+\.(htm|html))\"\s*>\s*(.*?)\s*</a>",rtext,re.DOTALL) :
-        url=m[1]
+        url=m[1].split("/")[-1]
         chapter=m[3]
         chapters.append((url,chapter))
-    chapters = sorted(list(set(chapters)))
+    def cid(c) :
+        s = c[0]
+        if not s :
+            return 2**31
+        s = re.sub(r"(\.html|\.htm)",'',s)
+        m = re.search(r"(\d+)",s)
+        if m :
+            return int(m.group(1))
+        else :
+            return 2**31
+    chapters = sorted(list(set(chapters)),key=cid)
     if debug :
         print(chapters)
     report=""
@@ -60,7 +81,7 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         if cdone > limit :
             break
         url=m[0]
-        cname = url.split("/")[-1]
+        cname = url
         # redownload the last chapter to fix the prev/next link
         if cname and cname < last :
             continue
@@ -69,16 +90,20 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         chapter=m[1]
         print("{:60}  ({})".format(url,chapter),file=sys.stderr,flush=True)
         try :
-            rsp = ss.get(url,timeout=20,verify=False)
+            user_agent = {'User-Agent': random_useragent()}
+            if debug :
+                print("# {}".format(url),file=sys.stderr,flush=True)
+                print("# {}".format(user_agent),file=sys.stderr,flush=True)
+            rsp = ss.get(url,headers=user_agent,timeout=20,verify=False)
         except :
            print("# {}".format(traceback.format_exc().splitlines()[-1]),file=sys.stderr,flush=True)
            break
         rsp.encoding = rsp.apparent_encoding
-        m = re.search(r"<div id=\"content\">(.*?)</div>",rsp.text,re.DOTALL)
+        m = re.search(r"<div (class|id)=\"content\".*?>(.*?)</div>",rsp.text,re.DOTALL)
         context = ""
         if m :
-            context = m.group(1).replace("</br>","</br>\n")
-            context = m.group(1).replace("<br/>","<br/>\n")
+            context = m.group(2).replace("</br>","</br>\n")
+            context = m.group(2).replace("<br/>","<br/>\n")
         else :
             print("# cannot find content", file=sys.stderr, flush=True)
             break
@@ -97,26 +122,27 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         if ix == len(chapters)-1 :
             nextpg = "index.html"
         else :
-            nextpg = chapters[ix+1][0].split("/")[-1]
+            nextpg = chapters[ix+1][0]
         rtext = "<html>\n"
-        rtext = "<head>\n"
+        rtext += "<head>\n"
         rtext += "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\">\n"
         rtext += '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>' + "\n"
         rtext += "<title>{}</title>\n".format(title+":"+chapter)
         ps = """<script type="text/javascript"> var preview_page = "prevpg"; var next_page = "nextpg"; var index_page = "index.html"; function jumpPage() { if (event.keyCode==37) location=preview_page; if (event.keyCode==39) location=next_page; if (event.keyCode==13) location=index_page; } document.onkeydown=jumpPage; </script> </head>\n"""
-        rtext = "</head>\n"
-        rtext += "<body style=\"background-color: #EAF2F8;color: #000000;\">\n"
         rtext += ps.replace("prevpg",prevpg).replace("nextpg",nextpg)
-        rtext += "<a href=\"{}\">PREV CHAPTER</a>\n".format(prevpg)
-        rtext += "&nbsp;"*10
+        rtext += "</head>\n"
+        rtext += "<body style=\"background-color: #EAF2F8;color: #000000;\">\n"
+        rtext += "<a href=\"{}\">PREV CHAPTER</a><br/><br/>\n".format(prevpg)
+        #rtext += "&nbsp;"*10
         rtext += "<a href=\"{}\">NEXT CHAPTER</a></br>\n".format(nextpg)
         rtext += "</br></br>\n"
         rtext += context + "\n"
         rtext += "</br></br>\n"
-        rtext += "<a href=\"{}\">PREV CHAPTER</a>\n".format(prevpg)
-        rtext += "&nbsp;"*10
-        rtext += "<a href=\"{}\">NEXT CHAPTER</a></br></br>\n".format(nextpg)
-        rtext += "<a href=\"{}\">orignal link</a></br>\n".format(url)
+        rtext += "<a href=\"{}\">PREV CHAPTER</a><br/><br/>\n".format(prevpg)
+        #rtext += "&nbsp;"*10
+        rtext += "<a href=\"{}\">NEXT CHAPTER</a><br/><br/><br/><br/>\n".format(nextpg)
+        rtext += "<a href=\"index.html\">index</a><br/><br/>\n"
+        rtext += "<a href=\"{}\">orignal link</a><br/>\n".format(url)
         rtext += "</body>\n"
         rtext += "</html>\n"
         with open(os.path.join(tgtdir,title, cname),"w") as f :
@@ -133,7 +159,7 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
                     report += ", {}".format(chapter)
             downloaded.add(cname)
         cdone+=1
-        time.sleep(1)
+        time.sleep(float(random.choice([i for i in range(30)])/10.0))
     report = report.rstrip(",")
     print("# updates : {}".format(report), file=sys.stderr, flush=True)
     return report
@@ -151,7 +177,7 @@ def batch_fetch(cfgfile,debug=False) :
                 if not thefirst :
                     print("# delay 30s", file=sys.stderr, flush=True)
                     thefirst = False
-                    time.sleep(30)
+                    time.sleep(20+random.choice([i for i in range(60)]))
                 title = m.group(1)
                 tgtdir = m.group(2)
                 page = m.group(3)
